==> psgconf-3.2.tar.gz: psgconf-3.2/lib/PSGConf/Action/GenerateFile/httpd_conf.pm <==

###
###  Copyright 2000-2006 University of Illinois Board of Trustees
###  All rights reserved. 
###
###  httpd_conf.pm - psgconf action module for generating httpd.conf
###
###  Campus Information Technologies and Educational Services
###  University of Illinois at Urbana-Champaign
###


package PSGConf::Action::GenerateFile::httpd_conf;

use strict;

use PSGConf::Action::GenerateFile;

our @ISA = qw(PSGConf::Action::GenerateFile);


###############################################################################
###  utility function to sort vhost addresses
###############################################################################

### sorting function for Apache address:port strings
sub _sort_www_addrs
{
	### inline sort function for sorting vhost declarations.
	### this makes sure that "default" vhosts (those with no server
	### name) are placed at the end of the file
	return 1 if ( $a !~ /:/ );
	return -1 if ( $b !~ /:/ );

	my ($a_addr, $a_port) = split(/:/, $a);
	my ($b_addr, $b_port) = split(/:/, $b);
	my $ret = ($a_addr cmp $b_addr);
	return ($ret == 0 ? $a_port <=> $b_port : $ret);
}


###############################################################################
###  generate() method
###############################################################################

sub generate
{
	my ($self, $fh, $psgconf) = @_;
	my ($opt, $addr, $name_vhost, $vhost, $ssl);
	my ($mod, $mod_add_list, $url, $port);
	my ($lref);

	print $fh <<EOF;

###
###  Global Environment
###

EOF

	if ( ! $self->{use_apache2} )  {
		print $fh "ServerType		standalone\n";
	}
	print $fh "ServerRoot		$self->{server_dir}\n";

	if ( ! $self->{use_apache2} )  {
		print $fh <<EOF;
ScoreBoardFile		$self->{pidfile_dir}/httpd.scoreboard
ResourceConfig		/dev/null
AccessConfig		/dev/null

EOF
	}

	foreach $opt (sort keys %{$self->{global_options}})
	{
		print $fh "$opt\t\t$self->{global_options}->{$opt}\n";
	}

	print $fh "\n";
	foreach $addr (sort _sort_www_addrs keys %{$self->{listen_addrs}})
	{
		$ssl = 0;
		foreach $vhost (keys %{$self->{virtual_hosts}})
		{
			$lref = $self->{virtual_hosts}->{$vhost};
			next
				if (! grep(/\b${addr}\b/, @$lref[1..$#$lref]));
			if (exists($self->{vh_ssl}->{$vhost}))
			{
				$ssl = 1;
				last;
			}
		}

		print $fh "<IfDefine SSL>\n"
			if ($ssl);
		print $fh "Listen\t\t\t$addr\n";
		print $fh "</IfDefine>\n"
			if ($ssl);
	}

	print $fh "\n$self->{global_config}\n"
		if (defined($self->{global_config}));

	print $fh <<EOF;

###
###  module configuration
###

EOF

	$mod_add_list = "\n";
	if ( ! $self->{use_apache2} )  {
		$mod_add_list = "ClearModuleList\n";
	}
	foreach $mod (@{$self->{modules}})
	{
		my ($modname, $modpath, $modadd, $modcond) = @$mod;

		next
			if ($modcond eq 'SSL'
			    && ! %{$self->{vh_ssl}});

		if ($modname ne '')
		{
			print $fh "<IfDefine $modcond>\n"
				if (defined($modcond));
			printf $fh "LoadModule %-25s %s\n",
				   "${modname}_module", $modpath;
			print $fh "</IfDefine>\n"
				if (defined($modcond));
		}

		if ( ! $self->{use_apache2} )  {
			$mod_add_list .= "<IfDefine $modcond>\n"
				if (defined($modcond));
			$mod_add_list .= "AddModule ${modadd}\n";
			$mod_add_list .= "</IfDefine>\n"
				if (defined($modcond));
		}
	}
	print $fh "\n$mod_add_list";

	print $fh <<EOF;

###
###  Main Server Config
###  (Default settings inheritted by other virtual hosts)
###

ServerName		$self->{main_server_name}
ServerAdmin		$self->{main_admin_alias}\@$self->{main_server_name}
User			$self->{user_name}
Group			$self->{group_name}
EOF

	if ( $self->{log_syslog} ) {
		print $fh "ErrorLog		syslog:$self->{log_fac}\n";
	} else {
		print $fh "ErrorLog		$self->{log_dir}/error_log\n";
	}

	print $fh <<EOF;
LogLevel		$self->{log_level}

DocumentRoot		$self->{document_dir}
<Directory $self->{document_dir}>
    Options Indexes FollowSymLinks ExecCGI
    AllowOverride $self->{default_allowoverride}
    Order allow,deny
    Allow from all
</Directory>

<IfModule mod_mime.c>
    TypesConfig		$self->{mime_dir}/mime.types
</IfModule>

<IfModule mod_mime_magic.c>
    MIMEMagicFile	$self->{config_dir}/magic
</IfModule>

<IfModule mod_alias.c>
    Alias /icons/ $self->{icons_dir}/
    <Directory $self->{icons_dir}>
        Options Indexes MultiViews
        AllowOverride None
        Order allow,deny
        Allow from all
    </Directory>

    ScriptAlias /cgi-bin/ $self->{server_dir}/cgi-bin/
    <Directory $self->{server_dir}/cgi-bin>
        Options None
        AllowOverride None
        Order allow,deny
        Allow from all
    </Directory>
</IfModule>

EOF

	foreach $opt (sort keys %{$self->{main_server_options}})
	{
		print $fh "$opt\t\t$self->{main_server_options}->{$opt}\n";
	}

	if (%{$self->{vh_ssl}})
	{
		print $fh <<EOF;

<IfDefine SSL>
LogFormat "%h %l %u %t \\"%r\\" %>s %b \\"%{Referer}i\\" \\"%{User-Agent}i\\" %{SSL_PROTOCOL}x %{SSL_CIPHER}x" ssl
</IfDefine>
EOF
	}

	print $fh <<EOF;

LogFormat "%h %l %u %t \\"%r\\" %>s %b \\"%{Referer}i\\" \\"%{User-Agent}i\\"" combined
CustomLog $self->{log_dir}/access_log combined

EOF

	if (defined($self->{main_server_config}))
	{
		print $fh "\n$self->{main_server_config}\n";
	}

	if (%{$self->{vh_ssl}})
	{
		print $fh <<EOF;

###
### SSL configuration
###

<IfDefine SSL>
AddType application/x-x509-ca-cert .crt
AddType application/x-pkcs7-crl    .crl
</IfDefine>

<IfModule mod_ssl.c>
SSLPassPhraseDialog	builtin
SSLSessionCache		dbm:$self->{pidfile_dir}/ssl_scache
SSLSessionCacheTimeout	300
SSLMutex		file:$self->{pidfile_dir}/ssl_mutex
SSLRandomSeed		startup builtin
SSLRandomSeed		connect builtin
EOF

		if ( ! $self->{use_apache2} )  {
			print $fh <<EOF;
SSLLog			$self->{log_dir}/ssl_engine_log
SSLLogLevel		$self->{ssl_log_level}
</IfModule>

EOF
		} else {
			if ( $self->{log_syslog} ) {
				print $fh "ErrorLog		syslog:$self->{log_fac}\n";
			} else {
				print $fh "ErrorLog		$self->{log_dir}/ssl_engine_log\n";
			}

			print $fh <<EOF;
LogLevel		$self->{ssl_log_level}
</IfModule>

EOF
		}
	}

	print $fh <<EOF;

###
###  Virtual Hosts
###

EOF

	map {
		print $fh "NameVirtualHost $_\n";
	} (sort _sort_www_addrs keys %{$self->{name_vhost_addrs}});

	foreach $vhost (sort _sort_www_addrs keys %{$self->{virtual_hosts}})
	{
		$self->_add_vhost($fh, $vhost, $psgconf);
	}

	print $fh "\n";
	return 1;
}


###############################################################################
###  utility function to add vhost sections
###############################################################################

sub _add_vhost
{
	my ($self, $fh, $vhost, $psgconf) = @_;
	my ($ssl, @vhostport, $server_name);

	print $fh "\n### virtual host \"$vhost\"";

	$ssl = 1
		if (exists($self->{vh_ssl}->{$vhost}));

	print $fh "\n<IfDefine SSL>"
		if ($ssl);

	($server_name, @vhostport) = @{$self->{virtual_hosts}->{$vhost}};
	print $fh "\n<VirtualHost " . join(' ', @vhostport) . ">\n";
	if (defined($server_name)
	    && $server_name ne '')
	{
		print $fh "    ServerName $server_name\n";
		print $fh "    ServerAdmin webmaster\@$server_name\n";
	}

	if (exists($self->{vh_document_root}->{$vhost}))
	{
		print $fh <<EOF;

    DocumentRoot $self->{vh_document_root}->{$vhost}
    <Directory $self->{vh_document_root}->{$vhost}>
        Options Indexes FollowSymLinks ExecCGI
        AllowOverride $self->{default_allowoverride}
        Order allow,deny
        Allow from all
    </Directory>
EOF
	}

	if ($ssl)
	{
		print $fh <<EOF;

    SSLEngine			on
    SSLCipherSuite		$self->{ssl_cipher_suite}
    SSLCertificateFile		$self->{config_dir}/ssl.crt/$server_name.crt
    SSLCertificateKeyFile	$self->{config_dir}/ssl.key/$server_name.key
    <Files ~ "\.(cgi|shtml|phtml|php3?)\$">
        SSLOptions +StdEnvVars
    </Files>
    <Directory "$self->{server_dir}/cgi-bin">
        SSLOptions +StdEnvVars
    </Directory>

    SetEnvIf User-Agent ".*MSIE.*" \\
             nokeepalive ssl-unclean-shutdown \\
             downgrade-1.0 force-response-1.0
EOF
	}

	if (exists($self->{vh_access_log}->{$vhost}))
	{
		print $fh "\n    CustomLog $self->{vh_access_log}->{$vhost} "
			  . ($ssl ? 'ssl' : 'combined')
			  . "\n";
	}

	$self->{vh_config}->{$vhost} =~ s/\@WWW_LOG_DIR\@/$self->{log_dir}/g;
	$self->{vh_config}->{$vhost} =~ s/\@WWW_SERVER_DIR\@/$self->{server_dir}/g;
	$self->{vh_config}->{$vhost} =~ s/\@WWW_CONFIG_DIR\@/$self->{config_dir}/g;
	print $fh $self->{vh_config}->{$vhost} . "\n"
		if (defined($self->{vh_config}->{$vhost}));
	print $fh "</VirtualHost>\n";

	print $fh "</IfDefine>\n"
		if ($ssl);
}


###############################################################################
###  documentation
###############################################################################

1;

__END__

=head1 NAME

PSGConf::Action::GenerateFile::httpd_conf - generate httpd.conf file

=head1 SYNOPSIS

  use PSGConf::Action::GenerateFile::httpd_conf;

  $psgconf->register_actions(
		PSGConf::Action::GenerateFile::httpd_conf->new(
			'name'		=> '/usr/local/etc/apache/httpd.conf',
			'server_dir'	=> '...',
			'pidfile_dir'	=> '...',
			'log_dir'	=> '...',
			'config_dir'	=> '...',
			'icons_dir'	=> '...',
			'global_options' => { ... },
			'global_config'	=> '...',
			'modules'	=> { ... },
			'main_server_name' => '...',
			'main_server_options' => { ... },
			'main_server_config' => '...',
			'listen_addrs'	=> [ ... ],
			'name_vhost_addrs' => [ ... ],
			'virtual_hosts'	=> { ... },
			'vh_ssl'	=> { ... },
			'vh_document_root' => { ... },
			'vh_access_log'	=> { ... },
			...
		),
		...
	);

=head1 DESCRIPTION

The B<PSGConf::Action::GenerateFile::httpd_conf> module provides a
B<PSGConf> action class for generating the Apache F<httpd.conf> file.

The B<PSGConf::Action::GenerateFile::httpd_conf> class is derived from the
B<PSGConf::Action::GenerateFile> class, but it defines/overrides the
following methods:

=over 4

=item generate()

Writes the value of the I<content> attribute to the output file.

=back

In addition to the attributes supported by
the B<PSGConf::Action::GenerateFile> class, the
B<PSGConf::Action::GenerateFile::httpd_conf> class supports the following
attributes:

=over 4

=item I<server_dir>

The absolute path to the server directory.  This is used for the
C<ServerRoot> option and the parent directory for the main
C</cgi-bin/> alias.

=item I<document_dir>

The absolute path to the document directory.  This is used for the
C<DocumentRoot> option.

=item I<pidfile_dir>

The absolute path to the pidfile directory.  This is used for the
C<ScoreBoardFile>, C<SSLSessionCache>, and C<SSLMutex> options.

=item I<log_dir>

The absolute path to the log directory.  This is used for the
C<ErrorLog> and C<SSLLog> options, and for the access log for the main
server.

=item I<log_level>

The log level for the error log file.

=item I<ssl_log_level>

The log level for the SSL log file.

=item I<user_name>

The login to run the httpd process as.

=item I<group_name>

The group to run the httpd process as.

=item I<main_admin_alias>

The email alias for admin mail.

=item I<config_dir>

The absolute path to the configuration directory.  This is used for the
C<TypesConfig>, C<MIMEMagicFile>, C<SSLCertificateFile>, and
C<SSLCertificateKeyFile> options.

=item I<icons_dir>

The absolute path to the icons directory.  This is used to create the
C</icons/> alias.

=item I<global_options>

A hash containing options and their values for the global configuration
section of F<httpd.conf>.

=item I<global_config>

Literal text to be added to the global configuration section of
F<httpd.conf>.

=item I<modules>

An array containing an entry for each module.  Each entry is a reference
to a list containing the module name (i.e., the first argument to
the C<LoadModule> directive, without the C<_module> suffix), path to
the module file, name to be used in the C<AddModule> directive, and an
optional conditional token that indicates that the module statements
should be wrapped in an C<E<lt>IfDefineE<gt>> block.

=item I<main_server_name>

The server name of the main server.

=item I<main_server_options>

A hash containing options and their values for the main server
configuration section of F<httpd.conf>.

=item I<main_server_config>

Literal text to be added to the main server configuration section of
F<httpd.conf>.

=item I<listen_addrs>

A list of IP addresses and port numbers to bind to.

=item I<name_vhost_addrs>

A list of IP addresses and port numbers that will use name-based virtual
hosting.

=item I<virtual_hosts>

A hash containing virtual host definitions.  The key is a label by which
that virtual host will be known, and the value is a list containing the
server name for that virtual host and one or more address/port
specifications for which the virtual host will answer queries.

=item I<vh_ssl>

A hash whose keys are the virtual hosts that serve HTTPS requests.

=item I<vh_document_root>

A hash that maps vhost labels to the document root for that vhost.

=item I<vh_access_log>

A hash that maps vhost labels to the access log for that vhost.

=back

=head1 SEE ALSO

L<perl>

L<PSGConf>

L<PSGConf::Action::GenerateFile>

=cut

==> psgconf-3.2.4.tar.gz: psgconf-3.2.4/lib/PSGConf/Action/GenerateFile/httpd_conf.pm <==

###
###  Copyright 2000-2007 University of Illinois Board of Trustees
###  All rights reserved. 
###
###  httpd_conf.pm - psgconf action module for generating httpd.conf
###
###  Campus Information Technologies and Educational Services
###  University of Illinois at Urbana-Champaign
###


package PSGConf::Action::GenerateFile::httpd_conf;

use strict;

use PSGConf::Action::GenerateFile;

our @ISA = qw(PSGConf::Action::GenerateFile);


###############################################################################
###  utility function to sort vhost addresses
###############################################################################

### sorting function for Apache address:port strings
sub _sort_www_addrs
{
	my ($a_addr, $a_port) = split(/:/, $a);
	my ($b_addr, $b_port) = split(/:/, $b);
	my $ret = ($a_addr cmp $b_addr);
	return ($ret == 0 ? $a_port <=> $b_port : $ret);
}


###############################################################################
###  generate() method
###############################################################################

sub generate
{
	my ($self, $fh, $psgconf) = @_;
	my ($opt, $addr, $name_vhost, $vhost, $ssl);
	my ($mod, $mod_add_list, $url, $port);
	my ($lref);

	print $fh <<EOF;

###
###  Global Environment
###

EOF

	if ( ! $self->{use_apache2} )  {
		print $fh "ServerType		standalone\n";
	}
	print $fh "ServerRoot		$self->{server_dir}\n";

	if ( ! $self->{use_apache2} )  {
		print $fh <<EOF;
ScoreBoardFile		$self->{pidfile_dir}/httpd.scoreboard
ResourceConfig		/dev/null
AccessConfig		/dev/null

EOF
	}

	foreach $opt (sort keys %{$self->{global_options}})
	{
		print $fh "$opt\t\t$self->{global_options}->{$opt}\n";
	}

	print $fh "\n";
	foreach $addr (sort _sort_www_addrs keys %{$self->{listen_addrs}})
	{
		$ssl = 0;
		foreach $vhost (keys %{$self->{virtual_hosts}})
		{
			$lref = $self->{virtual_hosts}->{$vhost};
			next
				if (! grep(/\b${addr}\b/, @$lref[1..$#$lref]));
			if (exists($self->{vh_ssl}->{$vhost}))
			{
				$ssl = 1;
				last;
			}
		}

		print $fh "<IfDefine SSL>\n"
			if ($ssl);
		print $fh "Listen\t\t\t$addr\n";
		print $fh "</IfDefine>\n"
			if ($ssl);
	}

	print $fh "\n$self->{global_config}\n"
		if (defined($self->{global_config}));

	print $fh <<EOF;

###
###  module configuration
###

EOF

	$mod_add_list = "\n";
	if ( ! $self->{use_apache2} )  {
		$mod_add_list = "ClearModuleList\n";
	}
	foreach $mod (@{$self->{modules}})
	{
		my ($modname, $modpath, $modadd, $modcond) = @$mod;

		next
			if ($modcond eq 'SSL'
			    && ! %{$self->{vh_ssl}});

		if ($modname ne '')
		{
			print $fh "<IfDefine $modcond>\n"
				if (defined($modcond));
			printf $fh "LoadModule %-25s %s\n",
				   "${modname}_module", $modpath;
			print $fh "</IfDefine>\n"
				if (defined($modcond));
		}

		if ( ! $self->{use_apache2} )  {
			$mod_add_list .= "<IfDefine $modcond>\n"
				if (defined($modcond));
			$mod_add_list .= "AddModule ${modadd}\n";
			$mod_add_list .= "</IfDefine>\n"
				if (defined($modcond));
		}
	}
	print $fh "\n$mod_add_list";

	print $fh <<EOF;

###
###  Main Server Config
###  (Default settings inheritted by other virtual hosts)
###

ServerName		$self->{main_server_name}
ServerAdmin		$self->{main_admin_alias}\@$self->{main_server_name}
User			$self->{user_name}
Group			$self->{group_name}
EOF

	if ( $self->{log_syslog} ) {
		print $fh "ErrorLog		syslog:$self->{log_fac}\n";
	} else {
		print $fh "ErrorLog		$self->{log_dir}/error_log\n";
	}

	print $fh <<EOF;
LogLevel		$self->{log_level}

DocumentRoot		$self->{document_dir}
<Directory $self->{document_dir}>
    Options Indexes FollowSymLinks ExecCGI
    AllowOverride $self->{default_allowoverride}
    Order allow,deny
    Allow from all
</Directory>

<IfModule mod_mime.c>
    TypesConfig		$self->{mime_dir}/mime.types
</IfModule>

<IfModule mod_mime_magic.c>
    MIMEMagicFile	$self->{config_dir}/magic
</IfModule>

<IfModule mod_alias.c>
    Alias /icons/ $self->{icons_dir}/
    <Directory $self->{icons_dir}>
        Options Indexes MultiViews
        AllowOverride None
        Order allow,deny
        Allow from all
    </Directory>

    ScriptAlias /cgi-bin/ $self->{server_dir}/cgi-bin/
    <Directory $self->{server_dir}/cgi-bin>
        Options None
        AllowOverride None
        Order allow,deny
        Allow from all
    </Directory>
</IfModule>

EOF

	foreach $opt (sort keys %{$self->{main_server_options}})
	{
		print $fh "$opt\t\t$self->{main_server_options}->{$opt}\n";
	}

	if (%{$self->{vh_ssl}})
	{
		print $fh <<EOF;

<IfDefine SSL>
LogFormat "%h %l %u %t \\"%r\\" %>s %b \\"%{Referer}i\\" \\"%{User-Agent}i\\" %{SSL_PROTOCOL}x %{SSL_CIPHER}x" ssl
</IfDefine>
EOF
	}

	print $fh <<EOF;

LogFormat "%h %l %u %t \\"%r\\" %>s %b \\"%{Referer}i\\" \\"%{User-Agent}i\\"" combined
CustomLog $self->{log_dir}/access_log combined

EOF

	if (defined($self->{main_server_config}))
	{
		print $fh "\n$self->{main_server_config}\n";
	}

	if (%{$self->{vh_ssl}})
	{
		print $fh <<EOF;

###
### SSL configuration
###

<IfDefine SSL>
AddType application/x-x509-ca-cert .crt
AddType application/x-pkcs7-crl    .crl
</IfDefine>

<IfModule mod_ssl.c>
SSLPassPhraseDialog	builtin
SSLSessionCache		dbm:$self->{pidfile_dir}/ssl_scache
SSLSessionCacheTimeout	300
SSLMutex		file:$self->{pidfile_dir}/ssl_mutex
SSLRandomSeed		startup builtin
SSLRandomSeed		connect builtin
EOF

		if ( ! $self->{use_apache2} )  {
			print $fh <<EOF;
SSLLog			$self->{log_dir}/ssl_engine_log
SSLLogLevel		$self->{ssl_log_level}
</IfModule>

EOF
		} else {
			if ( $self->{log_syslog} ) {
				print $fh "ErrorLog		syslog:$self->{log_fac}\n";
			} else {
				print $fh "ErrorLog		$self->{log_dir}/ssl_engine_log\n";
			}

			print $fh <<EOF;
LogLevel		$self->{ssl_log_level}
</IfModule>

EOF
		}
	}

	print $fh <<EOF;

###
###  Virtual Hosts
###

EOF

	map {
		print $fh "NameVirtualHost $_\n";
	} (sort _sort_www_addrs keys %{$self->{name_vhost_addrs}});

	### inline sort function for sorting vhost declarations.
	### this makes sure that "default" vhosts (those with no server
	### name) are placed at the end of the file
	sub _sort_vhosts
	{
		### Clear up make test, there is no real way to make
		### this work correctly.
		no warnings 'closure';
		if ($self->{virtual_hosts}->{$a}->[0] eq '')
		{
			return 1;
		}
		if ($self->{virtual_hosts}->{$b}->[0] eq '')
		{
			return -1;
		}
		return _sort_www_addrs($self->{virtual_hosts}->{$a}->[0],
							$self->{virtual_hosts}->{$b}->[0]);
	}

	foreach $vhost (sort _sort_vhosts keys %{$self->{virtual_hosts}})
	{
		$self->_add_vhost($fh, $vhost, $psgconf);
	}

	print $fh "\n";
	return 1;
}


###############################################################################
###  utility function to add vhost sections
###############################################################################

sub _add_vhost
{
	my ($self, $fh, $vhost, $psgconf) = @_;
	my ($ssl, @vhostport, $server_name);

	print $fh "\n### virtual host \"$vhost\"";

	$ssl = 1
		if (exists($self->{vh_ssl}->{$vhost}));

	print $fh "\n<IfDefine SSL>"
		if ($ssl);

	($server_name, @vhostport) = @{$self->{virtual_hosts}->{$vhost}};
	print $fh "\n<VirtualHost " . join(' ', @vhostport) . ">\n";
	if (defined($server_name)
	    && $server_name ne '')
	{
		print $fh "    ServerName $server_name\n";
		print $fh "    ServerAdmin webmaster\@$server_name\n";
	}

	if (exists($self->{vh_document_root}->{$vhost}))
	{
		print $fh <<EOF;

    DocumentRoot $self->{vh_document_root}->{$vhost}
    <Directory $self->{vh_document_root}->{$vhost}>
        Options Indexes FollowSymLinks ExecCGI
        AllowOverride $self->{default_allowoverride}
        Order allow,deny
        Allow from all
    </Directory>
EOF
	}

	if ($ssl)
	{
		print $fh <<EOF;

    SSLEngine			on
    SSLCipherSuite		$self->{ssl_cipher_suite}
    SSLCertificateFile		$self->{config_dir}/ssl.crt/$server_name.crt
    SSLCertificateKeyFile	$self->{config_dir}/ssl.key/$server_name.key
    <Files ~ "\.(cgi|shtml|phtml|php3?)\$">
        SSLOptions +StdEnvVars
    </Files>
    <Directory "$self->{server_dir}/cgi-bin">
        SSLOptions +StdEnvVars
    </Directory>

    SetEnvIf User-Agent ".*MSIE.*" \\
             nokeepalive ssl-unclean-shutdown \\
             downgrade-1.0 force-response-1.0
EOF
	}

	if (exists($self->{vh_access_log}->{$vhost}))
	{
		print $fh "\n    CustomLog $self->{vh_access_log}->{$vhost} "
			  . ($ssl ? 'ssl' : 'combined')
			  . "\n";
	}

	$self->{vh_config}->{$vhost} =~ s/\@WWW_LOG_DIR\@/$self->{log_dir}/g;
	$self->{vh_config}->{$vhost} =~ s/\@WWW_SERVER_DIR\@/$self->{server_dir}/g;
	$self->{vh_config}->{$vhost} =~ s/\@WWW_CONFIG_DIR\@/$self->{config_dir}/g;
	print $fh $self->{vh_config}->{$vhost} . "\n"
		if (defined($self->{vh_config}->{$vhost}));
	print $fh "</VirtualHost>\n";

	print $fh "</IfDefine>\n"
		if ($ssl);
}


###############################################################################
###  documentation
###############################################################################

1;

__END__

=head1 NAME

PSGConf::Action::GenerateFile::httpd_conf - generate httpd.conf file

=head1 SYNOPSIS

  use PSGConf::Action::GenerateFile::httpd_conf;

  $psgconf->register_actions(
		PSGConf::Action::GenerateFile::httpd_conf->new(
			'name'		=> '/usr/local/etc/apache/httpd.conf',
			'server_dir'	=> '...',
			'pidfile_dir'	=> '...',
			'log_dir'	=> '...',
			'config_dir'	=> '...',
			'icons_dir'	=> '...',
			'global_options' => { ... },
			'global_config'	=> '...',
			'modules'	=> { ... },
			'main_server_name' => '...',
			'main_server_options' => { ... },
			'main_server_config' => '...',
			'listen_addrs'	=> [ ... ],
			'name_vhost_addrs' => [ ... ],
			'virtual_hosts'	=> { ... },
			'vh_ssl'	=> { ... },
			'vh_document_root' => { ... },
			'vh_access_log'	=> { ... },
			...
		),
		...
	);

=head1 DESCRIPTION

The B<PSGConf::Action::GenerateFile::httpd_conf> module provides a
B<PSGConf> action class for generating the Apache F<httpd.conf> file.

The B<PSGConf::Action::GenerateFile::httpd_conf> class is derived from the
B<PSGConf::Action::GenerateFile> class, but it defines/overrides the
following methods:

=over 4

=item generate()

Writes the value of the I<content> attribute to the output file.

=back

In addition to the attributes supported by
the B<PSGConf::Action::GenerateFile> class, the
B<PSGConf::Action::GenerateFile::httpd_conf> class supports the following
attributes:

=over 4

=item I<server_dir>

The absolute path to the server directory.  This is used for the
C<ServerRoot> option and the parent directory for the main
C</cgi-bin/> alias.

=item I<document_dir>

The absolute path to the document directory.  This is used for the
C<DocumentRoot> option.

=item I<pidfile_dir>

The absolute path to the pidfile directory.  This is used for the
C<ScoreBoardFile>, C<SSLSessionCache>, and C<SSLMutex> options.

=item I<log_dir>

The absolute path to the log directory.  This is used for the
C<ErrorLog> and C<SSLLog> options, and for the access log for the main
server.

=item I<log_level>

The log level for the error log file.

=item I<ssl_log_level>

The log level for the SSL log file.

=item I<user_name>

The login to run the httpd process as.

=item I<group_name>

The group to run the httpd process as.

=item I<main_admin_alias>

The email alias for admin mail.

=item I<config_dir>

The absolute path to the configuration directory.  This is used for the
C<TypesConfig>, C<MIMEMagicFile>, C<SSLCertificateFile>, and
C<SSLCertificateKeyFile> options.

=item I<icons_dir>

The absolute path to the icons directory.  This is used to create the
C</icons/> alias.

=item I<global_options>

A hash containing options and their values for the global configuration
section of F<httpd.conf>.

=item I<global_config>

Literal text to be added to the global configuration section of
F<httpd.conf>.

=item I<modules>

An array containing an entry for each module.  Each entry is a reference
to a list containing the module name (i.e., the first argument to
the C<LoadModule> directive, without the C<_module> suffix), path to
the module file, name to be used in the C<AddModule> directive, and an
optional conditional token that indicates that the module statements
should be wrapped in an C<E<lt>IfDefineE<gt>> block.

=item I<main_server_name>

The server name of the main server.

=item I<main_server_options>

A hash containing options and their values for the main server
configuration section of F<httpd.conf>.

=item I<main_server_config>

Literal text to be added to the main server configuration section of
F<httpd.conf>.

=item I<listen_addrs>

A list of IP addresses and port numbers to bind to.

=item I<name_vhost_addrs>

A list of IP addresses and port numbers that will use name-based virtual
hosting.

=item I<virtual_hosts>

A hash containing virtual host definitions.  The key is a label by which
that virtual host will be known, and the value is a list containing the
server name for that virtual host and one or more address/port
specifications for which the virtual host will answer queries.

=item I<vh_ssl>

A hash whose keys are the virtual hosts that serve HTTPS requests.

=item I<vh_document_root>

A hash that maps vhost labels to the document root for that vhost.

=item I<vh_access_log>

A hash that maps vhost labels to the access log for that vhost.

=back

=head1 SEE ALSO

L<perl>

L<PSGConf>

L<PSGConf::Action::GenerateFile>

=cut
